
#include <UDPSocket.h>
#include <iostream>
#include <string>
#include <vector>
using namespace std;

char *me;

const int def_port = 42263;
const int def_rate = 256;
const int def_sec = 10;

void usage(){
  cerr << "Usage: " << me << "  [-p <port>] \n" ;
  cerr << " -p: port, default= " << def_port << "\n";
  exit(1);
}

double tim(void) {
  struct timeval tv;
  double t;

  gettimeofday(&tv, NULL);
  t = (double)tv.tv_sec + (double)tv.tv_usec/1000000.0;

  return t;
}
  
void my_usleep(u_int64_t usec) {
  double now, till;
  int n;
  /* The current time */
  now = tim();

  /* The time we are sleeping till */
  till = now+usec/1.0e6;

  /* and spin for the rest of the time */
  n = 0;
  while (now<till) {
    now = tim();
    n++;
  }
  
}


main(int argc, char *argv[]){
  me = (me = strrchr(argv[0], '/')) == NULL ? argv[0] : me+1; /* My name */ 
  
  int k = 0; 
  int port = def_port;
  int rate = def_rate;
  int sec = def_sec;
  string hostname = "localhost";
  
  while (k >= 0) { 
    switch (k = getopt(argc, argv, "hp:a:r:l:")) { 
    case 'h':   usage();     break;
    case 'p':   port = atoi(optarg);      break;
    case 'r':   rate = atoi(optarg);      break;
    case 'l':   sec = atoi(optarg);      break;
    case 'a':   hostname = optarg;      break;
    case '?':    usage(); break;
    }
  }

  int mtu=1500;

  Socket *sock_ = new UDPSocket(mtu);
  
  const int pkt_size = 1000;
  vector <uint8_t>  packet(pkt_size);
  int ret = sock_->connect(hostname,port);
  if(ret <0) {
    cerr << "Error in connection to " << hostname << ":" << port << endl;
    exit(1);
  }

  uint32_t npkt = (uint32_t)((double)rate*1000000/pkt_size);
  double dt = 1./npkt;
  double t_start = tim();
  double t_startp1 = t_start + 1;
  double t_tmp = t_start+dt;
  int64_t usec;
  uint64_t total_byte(0), byte_sec(0);
  
  fprintf(stderr," Tx rate  total Byte \n");
  for(int isec =0; isec < sec ; isec++,t_startp1++){
    byte_sec=0;
    for(uint32_t i=0; i<npkt; i++, t_tmp+=dt){
      int ret = sock_->send(&packet[0], pkt_size);
      switch(ret){
      case -1:
	cerr << "Error in sending packet\n"; 
	exit(1); break;
      case 0:
	cerr << " 0 Byte written\n";
	break;
      default: 
	byte_sec += ret;
	break;
      }
      usec = (int64_t)((t_tmp - tim())*1000000);
      if(usec >0) my_usleep(usec);
    }
    fprintf(stderr," Tx rate (Mbps)  total Byte \n");
    fprintf(stderr,"   %7.3lf          %8ld \n", 
	     byte_sec/1000000*8, total_byte/1000000);

    usec = (int64_t)((t_startp1-tim())*1000000);
    if(usec >0) my_usleep(usec);
    t_tmp = t_startp1+dt;
  }
  
  delete sock_;

}

