/** 
 * @file  vsi-chdcon-srv.h
 * @brief Master and Dummy data server Class Implementation
 *
 * @author M. Sekido (sekido@nict.go.jp)
 * @date 2007/1/17
 * @version $Id: vsi-chdcon-srv.cpp,v 1.1.1.1 2009/03/24 01:53:04 sekido Exp $
 *
 * Copyright (C) 2007 NICT All rights reserved.
 */

/** \mainpage
 * This is a software for extracting paticular channel's data and 
 * distribute it to multiple clients via TCP/IP.
 * Data source is supposed to be vsi2000_sender as output of 
 * ADS1000/2000 sampler. 
 *
 * Programs:
 * - vsi-chdcon-srv: Extracting paticular channel and distribute
 * - vsi-dstrbtr-srv: Distribute data from single source to multiple clients. 
 *
 * Main classes:
 * - Mst_Ch_Data_Server :  Providing data to a master client
 * - Dmy_Ch_Data_Server :  Prividing data to multile other clients.
 *
 */

#include "payload.h"
#include "vsi-chdcon.h"
#include "vsi-chdcon-srv.h"
#include "netutl.h"

void Usage(){
  cerr << "Usage: " << me 
       << "[-a <server ip>] [-p <srv port>] ";
  cerr << "  [-c <channels>] [-h] [-q <listening mstr port>]\n";
  cerr << "  [-d <listening dmy port>] [-e <dbg level>] [-t <This ip>]\n";
  cerr << "  [-r <Socket Rcv Bufsize>] [-s <Socket Snd Bufsize>]" << endl; 
  cerr << " Receiving VSI2000 data from server and dividing into channels\n"; 
  cerr << " Assuming the soruce data is 2 bit for 16 channels.\n"; 
  cerr << " -c: Selecting single channel for sending data to client.\n";
  cerr << "      ch=(0,1,2...)  default=0 (first)\n";
  cerr << " -q: Listen port to accept socket connection from MASTER client" 
       << endl;
  cerr << "     Default is  7002\n"; 
  cerr << " -d: Listen port to accept socket connection from DUMMY client\n"; 
  cerr << "     Default is  7003\n"; 
  cerr << " -h: Show this help" << endl;
  cerr << " -e:  less output <----0 1-----> More output" << endl;
  cerr << "  Note: Parameters <channel>, <sample>, are re-set by accepting";
  cerr << " config pcket from Master client" << endl;
  exit(1);
}

// Binding and Listening
// int Listener(Socket* sock, int port);
char* me;
int debug_level=0;

int  main(int argc, char *argv[]) {
 /* *** Initialize *** */
  me = (me = strrchr(argv[0], '/')) == NULL ? argv[0] : me+1; /* My name */ 
  unsigned short srv_port=7000;
  int mtu=1500;
  string srv_ip("localhost");
  int ich =0;
  unsigned long Msps=2;              // Default 2Msps
  u_int32_t mask_dat=0xc;            // Default 2nd channel
  unsigned short  mstr_clt_port=7002;// Default Master Client port
  unsigned short  dmy_clt_port =7003;// Default Dummy  Client port
  string this_ip;
  long so_sBufSize=16000000, so_rBufSize=16000000;

  /* * Parse command-line options, if any * */ 
  int k = 0; 
  while (k >= 0) { 
    switch (k = getopt(argc, argv, "hp:a:c:s:r:d:q:e:t:")) { 
    case 'h':   Usage();     break;
    case 'p':   srv_port = atoi(optarg);      break;
    case 'q':   mstr_clt_port = atoi(optarg);      break;
    case 'd':   dmy_clt_port = atoi(optarg);      break;
    case 'e':   debug_level = atoi(optarg);      break;
    case 'a':   srv_ip = optarg;     break;
    case 't':   this_ip = optarg;     break;
    case 'c':   
      ich = atoi(optarg); 
      if( ich < 0 || ich > 15) Usage();
      else mask_dat = mask[ich];     break;
    case 's':    so_sBufSize = atoi(optarg);     break;
    case 'r':    so_rBufSize = atoi(optarg);     break;
    case '?':    Usage(); break;
    }
  }
  
  
  // Setup Receiver 
//   Socket *in_sock = new TCPSocket(mtu);
//   // Bind
//   char name[80];
//   gethostname(name,80);
//   if(in_sock->bind(name,0) == -1){
//     fprintf(stderr,"%s in_sock->bind(): Error\n",me);
//     in_sock->close();
//     exit(1);
//   }
//   Recv_mem recv_clt(*in_sock);
  Recv_mem recv_clt(mtu);
  recv_clt.Set_server(srv_ip); // sps
  recv_clt.Set_sample_dur(Msps*1000000,10); // sps, sec
  recv_clt.Set_Srv_port(srv_port); // sps
  recv_clt.Set_bitmask(mask_dat); 
  recv_clt.Set_RecvBufSize(1024); 
  recv_clt.Set_so_rBufSize(so_rBufSize); 
  
  // Setup Server for data distributuon 
  Socket *out_sock1 = new TCPSocket(mtu);
  Socket *out_sock2 = new TCPSocket(mtu);
  fprintf(stderr,"reuse=%d \n", out_sock1->get_so_reuseaddr());
  fprintf(stderr,"reuse=%d \n", out_sock2->get_so_reuseaddr());
  if(Listener(out_sock1, mstr_clt_port,this_ip)<0) exit(1);
  if(Listener(out_sock2, dmy_clt_port,this_ip) <0) exit(1);
  out_sock1->set_so_sndbuf(so_sBufSize);
  out_sock2->set_so_sndbuf(so_sBufSize);
  Mst_Ch_Data_Server mst_dat_srv(*out_sock1);
  Dmy_Ch_Data_Server dmy_dat_srv(*out_sock2);
  dmy_dat_srv.Set_so_sBufSize(so_sBufSize);
  dmy_dat_srv.Set_Master(&mst_dat_srv);
  mst_dat_srv.Set_Receiver(&recv_clt); // Set Data receiver from data src 
  
  signal(SIGINT, SIG_IGN);
  // Server start
  //  mst_dat_srv.proc();
  //  mst_dat_srv.join();
  //  mst_dat_srv.thread_func();
  dmy_dat_srv.proc();
  int status =1;
  // status = dmy_dat_srv.join();
  // fprintf(stderr, "Return from dmy_dat_srv=%d\n", status);
  //  Read_out(recv_clt,10);


  char sRecvBuf[256];
  while(status){
    fprintf(stderr,"Terminate by \"QUIT\" \n");
    scanf("%s", sRecvBuf);
    StrUpper(sRecvBuf,strlen(sRecvBuf));
    if(strncmp(sRecvBuf,"QUIT",4)==0){
      break;
    }
  }
  
  //  recv_clt.Rm_shm();
  
  //  recv_clt.stop();
  //  recv_clt.join();
  //  delete in_sock;
  fprintf(stderr,"delete outsock1\n");
  delete out_sock1;  
  fprintf(stderr,"delete outsock2\n");
  delete out_sock2;
  fprintf(stderr,"Finished\n");
}


// const int Read_out(Recv_mem & recv_clt, int nrpt){
//   //  usleep(50);
//   for(int cnt=0; cnt<nrpt; cnt++){
//     if(recv_clt.Lock_forRead(cnt % 2) != 0){
//       fprintf(stderr,"Memory page (%d)is not available\n", cnt%2);
//       return -1;
//     }
//     u_int8_t* rdadr = recv_clt.Get_Rpoint();
//     if(rdadr ==0){
//       fprintf(stderr,"Read address is not ready\n");
//       return -1;
//     }
//     fprintf(stderr, "  Read Out data %2d-------\n", cnt);
//     for(int j=0; j<4; j++){
//       for(int i=0; i<4;i++,rdadr+=4){
// 	fprintf(stderr, "%x%x%x%x ",
// 		*rdadr, *(rdadr+1), *(rdadr+2), *(rdadr+3) );
//       }
//       fputc('\n',stderr);
//     }
//   }
//   return 0;
// }
