/** 
 * @file  vsi-chdcon-srv.h
 * @brief Master and Dummy data server Class Implementation
 *
 * @author M. Sekido (sekido@nict.go.jp)
 * @date 2007/1/17
 * @version $Id: vsi-chdcon-srv.h,v 1.1.1.1 2009/03/24 01:53:04 sekido Exp $
 *
 * Copyright (C) 2007 NICT All rights reserved.
 */


// Channel Decomposition software server module 
#ifndef __VSI_CHDCON_SRV__
#define __VSI_CHDCON_SRV__

#include "netutl.h"


const int Max_Dmy_Client = 5;

extern char *me;
extern int debug_level;

const int Get_myport(Socket& s);


/** VSI data channel decomposer server 
 * @doc This headder file contains Class of Mst_Data_Server 
 * and Dmy_Ch_Data_Server. These are server object for 
 * data provision throough the TCP/IP.
 * All Member functions are writtern as inline functions.
 * @author Mamoru Sekido (sekido@nict.go.jp)
*/
//-----------------------------------------------------------------
//// Server Class cor Master Client 
class Mst_Ch_Data_Server:public Thread{
  Socket& _out_sock1;
  Socket* _out_sock2[Max_Dmy_Client];
  Recv_mem* recv_mem;
  int dmy_clt_cnt;
  static int nClientID;
 public:
  Mst_Ch_Data_Server(Socket &s1);
  /// Destructor 
  ~Mst_Ch_Data_Server(){
    Close_sockets();  // if(_out_sock1.get_sockd() != -1) _out_sock1.close();
  }
  void* thread_func(void);
  /// A function starting the thread
  void proc(){start();}
  int Add_dmyClnt(Socket* s);
  /// Remove the entry of dummy client
  void Rm_dmyClnt(int id){if (_out_sock2[id]!=0) _out_sock2[id]=0;}
  /// Set data receiver 
  void Set_Receiver( Recv_mem* rv){recv_mem = rv;}
  int CommunicateWithClient();
  int Dist_start(Socket& sock, const int); // Data Provider 
  void Close_sockets();
  /// Return number of dumy clients.
  const int NDmyClient()const {return nClientID;}
};
int Mst_Ch_Data_Server::nClientID = 0;

/// Constructor 
inline Mst_Ch_Data_Server::Mst_Ch_Data_Server(Socket &s1): 
  _out_sock1(s1),dmy_clt_cnt(0)
{
  for(int i=0; i<Max_Dmy_Client; i++){
    _out_sock2[i]=0;
  }
}

/// ߡ() 饤Ȥɲ
inline int Mst_Ch_Data_Server::Add_dmyClnt(Socket* s){
  // Check the number 
  int n=0;
  for(int i=0;i<Max_Dmy_Client; i++)
    if(_out_sock2[i]!=0) n++;
  dmy_clt_cnt =n;
  
  if( dmy_clt_cnt >= Max_Dmy_Client){
    fprintf(stderr,"Client Too Much (already %d), Connection refused.\n",
	    dmy_clt_cnt);
    if(s->get_sockd() != -1) s->close();
    delete s;
    return -1;
  }

  int i=0;
  while(i<Max_Dmy_Client && _out_sock2[i]!=0) i++;
  if( i==Max_Dmy_Client){
    if(s->get_sockd() != -1) s->close();
    delete s; return -1;
  }
  _out_sock2[i]=s;
  dmy_clt_cnt++;
  
  return dmy_clt_cnt;
}


/// ޥ() 饤 Thread ؿ
inline void* Mst_Ch_Data_Server::thread_func(void){

  int ret=0;
  if(nClientID !=0){
    fprintf(stderr,"Mst_Ch_Data_Server is already running\n");
  }else{
    nClientID++; 
    while( (ret=CommunicateWithClient()) ==-1);
    nClientID--;
  }
  if(ret!=-2) 
    proc(); // Generate other thread for next connection

}

/// Server communication module with client 
inline int Mst_Ch_Data_Server::CommunicateWithClient(){
  static int err_cntr=0;
  int ThreadSock;
  Socket* tcp_sock;
  if(err_cntr>10) return -2;

  fprintf(stderr,"Waiting for connection from Mstr Clnt...");
  if( (ThreadSock = _out_sock1.accept()) == -1){
    fprintf(stderr,"Error in MST Srv Accept (errno=%d) \n", errno);
    perror("Accept Error");
    _out_sock1.Error();
    err_cntr++;
    return -1;
  }else{
    fprintf(stderr, "accepted\n");
    tcp_sock = new TCPSocket(ThreadSock,_out_sock1.get_mtu());
    tcp_sock->Print();
  }
  
  // Receive request and payload-config  
  Payload payld;
  long status;
  if((status = tcp_sock->recv((u_int8_t *)&payld, sizeof(payld))) <=0){
    if(status ==0){
      fprintf(stderr,"Connection is already closed\n");
      err_cntr++;
      return -1;
    }else if(status == -1){
      fprintf(stderr,"Error in recv() (errno=%d):%s \n", 
	      errno, strerror(errno));
      err_cntr++;
      return -1;
    }
  }
  if(status != sizeof(payld)){
    fprintf(stderr,"%s :Error received size(%d) < Expecting (%d)\n",
	    me, status,  sizeof(payld)); 
    err_cntr++;
    return -1;
  } 
  fprintf(stderr,"Payload Contents for Client----\n");
  payld.Print();
  fprintf(stderr,"-------------------------------\n");
  recv_mem->Set_payload(payld);
  // Data Receiver start 
  //  recv_mem->Check_memsize();
  recv_mem->proc();
  
  // Data Provider start
  int dur = payld.length/payld.clock;
  Dist_start(*tcp_sock, dur);
  recv_mem->Set_ApplState(STOP_STATE);
  recv_mem->join();
  Close_sockets();
  if(tcp_sock->get_sockd() != -1) tcp_sock->close();
  delete tcp_sock;
  err_cntr=0;
  
  return 0;
}

/// Data Distributer 
inline int Mst_Ch_Data_Server::Dist_start(Socket &sock, const int dur){

  long long data_size=0;
  while(recv_mem->Get_Pagesize()==0) usleep(1);
  data_size= recv_mem->Get_Pagesize(); 
  unsigned long long total_byte=0;
  long status;

  fprintf(stderr,"Dist_start begin, data_size=%Ld\n", data_size);
  signal(SIGPIPE, SIG_IGN);
  int ipage=-1;
  for(int  isec=0; isec<dur; isec++){
    ipage = isec % 2;
    status = recv_mem->Check_forRead(ipage, &sock);
    switch(status){
    case -1:
      fprintf(stderr,"Memory page (%d)is not available\n", isec%2);
      return -1; break;
    case -2:
      fprintf(stderr,"Send out last once\n");
     isec=dur; break;
    case -3:
      fprintf(stderr,"Finish Immediately\n");
      signal(SIGPIPE, SIG_DFL);
      return -1; break;
    }
    u_int8_t* rdadr = (u_int8_t*)recv_mem->Get_page_addr(ipage);
    //    u_int8_t* wradr = recv_mem->Get_Rpoint();
    if(rdadr ==0){
      fprintf(stderr,"Read address is not ready\n");
      return -1;
    }

    long long remain_data =data_size;
    // Sending data to Master Client
    if( debug_level >0)
      fprintf(stderr,"Sending data to Master Client\n");

    while(remain_data >0){
      status = sock.send(rdadr, Min(data_size,remain_data));
      fprintf(stderr,"%.3f MB Sent\n", (float)status*1.e-6);
      if(status < 0){
	fprintf(stderr, "%s Error in sending data to Master Client\n",me);
	return -1;
      }else if(status ==0){
	fprintf(stderr, "%s Master Client is already closed.\n",me);
	//      return -1;
	continue;
      }
      remain_data-=status;
      rdadr+=status;
    }
    total_byte += data_size;
    // Sending data to Dumy Client

    for(int i=0; i<dmy_clt_cnt; i++){
      rdadr = (u_int8_t*)recv_mem->Get_Rpoint();
      if(_out_sock2[i] == 0) continue;
      remain_data =data_size;
      int err_cnt=0;
      while(remain_data > 0){
	status = _out_sock2[i]->send(rdadr, Min(data_size,remain_data));
	if(status < 0){
	  fprintf(stderr, "%s Error in sending data to Dmy Client\n",me);
	  fprintf(stderr, "Socket[%d] is destroyed,\n",i);
	  perror("send to Dmy socket");
	  if( _out_sock2[i]->get_sockd() !=-1) _out_sock2[i]->close();
	  delete _out_sock2[i];
	  _out_sock2[i]=0;
	  break;
	  //	  return -1;
	}else if(status ==0){
	  if(err_cnt < 10) {err_cnt++; continue;}
	  else{
	    fprintf(stderr, "%s Dmy Client is already closed.\n",me);
	    fprintf(stderr, "Socket[%d] is destroyed,\n",i);
	    if( _out_sock2[i]->get_sockd() !=-1)  _out_sock2[i]->close();
	    delete _out_sock2[i];
	    _out_sock2[i]=0;
	    //	    return -1;
	    break;
	  }
	} 
	err_cnt=0;
	rdadr += status;
	remain_data -= status;
      } // while loop
    } // for dmy
    status = recv_mem->Read_finsh(ipage);
  }// for sec
  
  signal(SIGPIPE, SIG_DFL);
  fprintf(stderr,"Total (%.3f MB) sent Succesfully\n",
	  (float)total_byte*1.e-6);
  return 0;
}

/// Sockets Υ
inline void  Mst_Ch_Data_Server::Close_sockets(){
  //  _out_sock1.close();
  for(int i=0; i<Max_Dmy_Client; i++)
    if( _out_sock2[i] !=0){
      if( _out_sock2[i]->get_sockd() !=-1)  _out_sock2[i]->close();
      delete _out_sock2[i];
      _out_sock2[i]=0;
    }
  dmy_clt_cnt=0;
  fprintf(stderr,"Close_sockets finished\n");
}

//-----------------------------------------------------------------
//// Server Class cor Dumy Client 
class Dmy_Ch_Data_Server:public Thread{
  Socket& _out_sock;
  static int nClientID;
  Mst_Ch_Data_Server* mst_srv;
  int sBufSize;
 public:
  /// Constructor 
  Dmy_Ch_Data_Server(Socket& s):_out_sock(s),mst_srv(0),sBufSize(1000000){;}
  /// Destructor
  ~Dmy_Ch_Data_Server(){ //if(_out_sock.get_sockd() != -1) _out_sock.close();
  }
  void* thread_func(void);
  /// A function starting the thread 
  void proc(){start();}
  /// Set Master data server
  void Set_Master(Mst_Ch_Data_Server* mst){mst_srv = mst;}
  /// Set Socket Buffer size 
  void Set_so_sBufSize(const int sb){sBufSize=sb;}
};
int Dmy_Ch_Data_Server::nClientID = 0;


/// ߡ()饤 Thread 
inline void* Dmy_Ch_Data_Server::thread_func(void){

  // Master data server start
  mst_srv->proc();
  
  if(nClientID !=0){
    fprintf(stderr,"Dmy_Ch_Data_Server is already running\n");
  }else{
    nClientID++;
    
    int mtu = _out_sock.get_mtu();
    int ThreadSock;
    Payload pld;
    while(1){
      if( (ThreadSock = _out_sock.accept()) == -1){
	fprintf(stderr,"Error in DMY Srv Accept (errno=%d) \n", errno);
	perror("Accept Error");
	_out_sock.Error();
	//    Error_accept();
	return (void*)-1;
      }
      fprintf(stderr, "Connection(%d) accepted by Dmy Client\n",nClientID);

      if( mst_srv->NDmyClient() >= Max_Dmy_Client) continue;
      
      Socket* tcp_sock = new TCPSocket(ThreadSock,mtu);
      tcp_sock->set_so_sndbuf(sBufSize);
      tcp_sock->Print();
      mst_srv->Add_dmyClnt(tcp_sock);

      int status = tcp_sock->recv((u_int8_t *)&pld, sizeof(pld));
      if( status == sizeof(pld)){
	fprintf(stderr,"Received Dmy payload is:");
	pld.Print();
      }

    }
    nClientID--;
  }
  
}





#endif
