/*
 *  Thread.cpp
 *  vrtp
 *
 *  Created by David Lapsley on Sun Feb 29 2004.
 *  Copyright (c) 2004 __MyCompanyName__. All rights reserved.
 *
 */
#include <cstdlib>
#include <cstdio>
#include <cstring>
#include <ctime>
#include <iostream>
using namespace std;

#include "Logsystem.hpp"
#include "Thread.h"

extern "C" 
{
#include <errno.h>
#include <syslog.h>
#include <pthread.h>
#include <signal.h>
    
    void* func(void* arg)
    {
        Thread* ptr=(Thread*)arg;
        ptr->thread_func();
        return(NULL);
    }  // End thread_func().
}

void* Thread::thread_func()
{
    syslog(LOG_DEBUG, "Thread::thread_func()");
    syslog(LOG_DEBUG, "-Thread::thread_func()");
    state = IDLE;
    return(NULL);
}  // End run_func().

Thread::Thread():
    state(IDLE),_thread_id(0),_id("anonymous")
{
  syslog(LOG_DEBUG, "Thread::Thread()");
  syslog(LOG_DEBUG, "-Thread::Thread()");
}  // End Thread().

Thread::Thread(const string& s):
    state(IDLE),_thread_id(0),_id(s)
{
  syslog(LOG_DEBUG, "Thread::Thread()");
  syslog(LOG_DEBUG, "-Thread::Thread()");
}  // End Thread().

Thread::~Thread(){
    syslog(LOG_DEBUG, "Thread::~Thread()");
    syslog(LOG_DEBUG, "-Thread::~Thread()");
}  // End ~Thread().

void Thread::start()
{
    syslog(LOG_DEBUG, "Thread::thread_create()");
    int ret=0;
    state = RUNNING;
    if ((ret=pthread_create(&_thread_id, NULL, &func, (void*)this))!=0) {
        syslog(LOG_ERR, "    Unable to initialize pthread: %s", strerror(ret));
    }
    // Important note... _thread_id may not be set before run_func() invocation.
    // Therefore, not a good idea to use _thread_id in the above. Use 
    // pthread_self() instead to identify a thread. This may/may not be an 
    // issue.
    syslog(LOG_DEBUG, "-Thread::thread_create()");
}  // End thread_create().

const int Thread::join(){
    
    if( _thread_id == 0){
      cerr << "Warning @Thread::join(): Thread("<< _id << ") is not running."
	   << endl;
	return 0;
    }
    syslog(LOG_DEBUG, "Thread::join()");
    syslog(LOG_DEBUG, "    pid: %s", _id.c_str());
    //    fprintf(stderr, "    pid: %s", _id.c_str());
    //    cerr << "Thread::join() pid:" << _id << endl; 
    int ret = pthread_join(_thread_id, NULL);
    switch(ret){
    case EINVAL:
      cerr << "E Thread::join(): Not a joinable thread\n";
      break;
    case ESRCH:
      cerr << "E Thread::join(): No thread could be found\n";
      break;
    case EDEADLK:
      cerr << "E Thread::join():Argument is using calling thread."
	   << strerror(errno) << endl; 
      break;
    default:
      break;
    }
    syslog(LOG_DEBUG, "Thread::join() failed\n");
    syslog(LOG_DEBUG, "-Thread::join()");
    state = IDLE;
    _thread_id=0;
    return ret;
}  // End join().


void Thread::stop(){
  syslog(LOG_DEBUG, "Thread::stop()");
  syslog(LOG_DEBUG, "    pid: %s", _id.c_str());
  cerr << "Thread():Stop()\n";
  Logsystem::info_logger()->Tstamp() << "Thread():Stop()\n";
  pthread_exit(NULL);
  syslog(LOG_DEBUG, "-Thread::stop()");
  state = IDLE;
  _thread_id=0;
}


const int Thread::cancel(){
  int ret= pthread_cancel(_thread_id);
  cerr << "Thead is being canceled.\n";
  state = ENDING;
  return ret;
}


const int Thread::detach(){
    syslog(LOG_DEBUG, "Thread::stop()");
    syslog(LOG_DEBUG, "    pid: %s", _id.c_str());
    int ret= pthread_detach(_thread_id);
    syslog(LOG_DEBUG, "-Thread::stop()");
    return ret;
}


const int Thread::send_sig(const int signo){
    if(_thread_id ==0){
	cerr << "Error in Thread::send_sig(): thread is not created/running." << endl;
	return -1;
    }
    
    int ret = pthread_kill(_thread_id, signo);
    switch(ret){
    case EINVAL:
	cerr << "Error in Thread::signal():Not a valid signal("
	     << signo << ")" << endl;
	return -1; 
	break;
    case ESRCH:
	cerr << "Error in Thread::signal(): This Thread is no more exist.\n";
	return -1; break;
    default:
	return ret;
    }
}


string Thread::StrStatus() const{
    string ret;
    switch(state){
    case IDLE:
	ret = "IDLE";  break;
    case RUNNING:
	ret = "RUNNING";  break;
    case ENDING:
	ret = "ENDING";  break;
    default:
	ret = "UN-KNOWN state";  break;
    }
    return ret;
}
