/* 
 * This software is released under the terms of the MIT License(included below). 
 *  
 * Copyright (c) 2003 MIT Haystack Observatory 
 *  
 * Permission is hereby granted, free of charge, to any person obtaining a   
 * copy of this software and associated documentation files (the "Software"),    
 * to deal in the Software without restriction, including without limitation   
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,   
 * and/or sell copies of the Software, and to permit persons to whom the  
 * Software is furnished to do so, subject to the following conditions: 
 *  
 * The above copyright notice and this permission notice shall be included in    
 * all copies or substantial portions of the Software. 
 *   
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,  
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE  
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER   
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
 * SOFTWARE. 
 * 
 */

#ifndef RTCPCLIENTSOCKET_H
#define RTCPCLIENTSOCKET_H

#include <common.h>
#include <RTCPSocket.h>
#include <string>
#include <iostream>
#include <RTPSession.h>
#include <Utils.h>

/** RTCPClientSocket class.
 * This class implements an RTP client socket. It inherits from TCPSocket.
 */

class RTCPClientSocket: public RTCPSocket
{
  public:
	 /** Constructor.
	  * This calls the base constructor to create an RTCPClientSocket.
	  */
	 RTCPClientSocket();
	 /** Constructor.
	  * This constructor creates a socket and sets the internal MTU member
	  * to be equal to the supplied parameter.
	  * @param s reference to a socket to be used for data transmission.
	  * @param rtp_session a reference to an RTPSession that will be used to
	  * store the protocol state.
	  * @param mtu MTU for packet transmission.
	  */
	 RTCPClientSocket(Socket& s,
							RTPSession& rtp_session,
							const int& mtu);
	 /** Copy constructor.
	  * Copies values of other VRClientSocket into this object.
	  * @param s VRClientSocket to copy.
	  */
	 RTCPClientSocket(const RTCPClientSocket &r);
	 /** Destructor. */
	 virtual ~RTCPClientSocket();
	 /** Socket API method.
	  * Attempt to establish a connection to the supplied address.
	  * @return 0: on success, -1 on failure.
	  */
    int connect();
	 /** Processing method.
	  * Recv RTCP compound packet from client side, process and store
	  * result in RTPSession structure.
	  */
	 int recv_proc();
	 /** Processing method.
	  * Send RTCP compound packet to client side, process and store
	  * result in RTPSession structure.
	  */
	 int send_proc();
};

ostream& operator<<(ostream& os, const RTCPClientSocket& r);

#endif


