#ifndef __PAYLOAD__
#define __PAYLOAD__

// This source code has written by Mamoru Sekido (sekido @ nict.go.jp) in Jan. 7002
// Anybody can use this software withour any warranty and under his/her own ressponsibility.
// Any feedback or comments to the author are welcome.
#include <string>
using namespace std;

extern "C" {
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdint.h>
#include <sys/types.h>
}

u_int32_t mask[16]=
  {0x3, 0xc, 0x30, 0xc0, 0x300, 0xc00, 0x3000, 0xc000,
   0x30000, 0xc0000, 0x300000, 0xc00000, 0x3000000, 
   0xc000000, 0x30000000, 0xc0000000};

/// For getter/sender comunication 
struct Payload {
  uint64_t    epoch;
  uint64_t   length; // bits/wire-stream/sec
  uint64_t   offset;
  uint32_t   clock; // Hz
  uint32_t   bit_mask; 
  public:
  Payload() :length(1),offset(0),clock(1),
	     bit_mask(0xc){epoch = (unsigned long long)time(0); }
  Payload(uint32_t mask):length(1),offset(0),clock(1),
			 bit_mask(mask){epoch = (unsigned long long)time(0); }
  // Copy Constractor 
  Payload(const Payload& pd):epoch(pd.epoch),length(pd.length),
			     offset(pd.offset),clock(pd.clock),
			     bit_mask(pd.bit_mask){ }
  void Print(FILE *fp=stderr);
  const int Nbits();
  const time_t Epoch()const {return epoch;}
  const time_t* aEpoch(time_t &t1)const { t1=(time_t)epoch; return(&t1);}
  const int Wait(const int t_width);
  const int Factor(){ 
    if(32 % Nbits() !=0){
      fprintf(stderr,"# of bits(%d) in Bit mask is not factor of 32\n",
	      Nbits());      return -1;    }
    return 32/Nbits();
  }
  const int Dur(){ return length/clock;}
  const uint32_t BitRate(){// Bitrate for one mask-stream
    return  (uint32_t)Nbits()* clock;  }
  const uint32_t BitMask() const{return bit_mask;}
  string Str_time();
  const time_t Stop_time(){ epoch + Dur();}
  const int Check();
} __attribute__ ((__packed__));


inline string Payload::Str_time(){
  char buf[80];
  time_t t1 = epoch;
  sprintf(buf,"%Y/%j-%H:%M:%S", gmtime(&t1));
  string s(buf);
  return s;
}

inline const int Payload::Check(){
  struct tm tm1;
  strptime("2000-001 00:00:00", "%Y-%j %H:%M:%S", &tm1);  
  time_t t0 = mktime( &tm1 );
  strptime("2100-001 00:00:00", "%Y-%j %H:%M:%S", &tm1);  
  time_t t1 = mktime(&tm1 );
  if(t0 > epoch || t1 < epoch) return -1; // Time is wrong 
  if(clock > 64000000 || clock < 1000000) return -2; // Clock is wrong 
  if( (32 % Nbits()) != 0)  return -3;  // mask is not appropriate.
  return 0;
}

inline const int Payload::Wait(const int dur){
  
  time_t t_now = time(0), t1=epoch;
  char time_code[80], time_code_st[80];  
  strftime(time_code_st,sizeof(time_code_st),"%Y/%j-%H:%M:%S",gmtime(&t1));
  if( epoch + dur < t_now){ 
    fprintf(stderr,"Error: Requested time is already passed\n");
    strftime(time_code,sizeof(time_code),"%Y/%j-%H:%M:%S",gmtime(&t_now));
    fprintf(stderr," Current time = %s\n",time_code);
    return -1;
  }
  
  if(epoch - 1 >= t_now){
    fprintf(stderr,"Waiting until start time %s\n", time_code_st);
    while( epoch -1 > time(0)){
      t_now = time(0);
      strftime(time_code,sizeof(time_code),"%Y/%j %H:%M:%S",gmtime(&t_now));
      fprintf(stderr,"\r Current time = %s  Target time=%s\r",time_code,time_code_st);
      usleep(300000);
    }
  }  
  return 0;
}


inline const int Payload::Nbits(){
  int cnt=0;
  unsigned int mask=bit_mask;
  for(int i=0; i<sizeof(bit_mask)*8; i++){
    if(mask & 0x01) cnt++;
    mask = mask >> 1;
  }
  return (cnt);
}

inline void Payload::Print(FILE *fp){
  char time_code[80];
  time_t t1;
  strftime(time_code,sizeof(time_code),"%Y/%j %H:%M:%S",gmtime(aEpoch(t1)));
  fprintf(fp,"Payload: Start time= %s (UTC) Dur= %ld sec\n", 
	  time_code, length/clock);
  fprintf(fp,"payload: Sampling Rate= %.3f MHz bit_mask = %0X\n",
	  (float)clock*1.e-6, bit_mask);
}

#endif
